"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = index;

var _fs = _interopRequireDefault(require("fs"));

var _path = _interopRequireDefault(require("path"));

var _mkdirp = _interopRequireDefault(require("mkdirp"));

var _normalizePath = _interopRequireDefault(require("normalize-path"));

var _loaderUtils = _interopRequireDefault(require("loader-utils"));

var _enhancedResolve = _interopRequireDefault(require("enhanced-resolve"));

var _findYarnWorkspaceRoot = _interopRequireDefault(require("find-yarn-workspace-root"));

var _cosmiconfig = _interopRequireDefault(require("cosmiconfig"));

var _babel = require("@linaria/babel");

var _logger = require("@linaria/logger");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * This file contains a Webpack loader for Linaria.
 * It uses the transform.ts function to generate class names from source code,
 * returns transformed code without template literals and attaches generated source maps
 */
const workspaceRoot = (0, _findYarnWorkspaceRoot.default)();
const lernaConfig = (0, _cosmiconfig.default)('lerna', {
  searchPlaces: ['lerna.json']
}).searchSync();
const lernaRoot = lernaConfig !== null ? _path.default.dirname(lernaConfig.filepath) : null;

function index(content, inputSourceMap) {
  var _this$_compilation, _result$sourceMap2;

  (0, _logger.debug)('loader', this.resourcePath);

  _babel.EvalCache.clearForFile(this.resourcePath);

  const {
    sourceMap = undefined,
    cacheDirectory = '.linaria-cache',
    preprocessor = undefined,
    extension = '.linaria.css',
    ...rest
  } = _loaderUtils.default.getOptions(this) || {};
  const root = workspaceRoot || lernaRoot || process.cwd();
  const baseOutputFileName = this.resourcePath.replace(/\.[^.]+$/, extension);
  const outputFilename = (0, _normalizePath.default)(_path.default.join(_path.default.isAbsolute(cacheDirectory) ? cacheDirectory : _path.default.join(process.cwd(), cacheDirectory), this.resourcePath.includes(root) ? _path.default.relative(root, baseOutputFileName) : baseOutputFileName));
  const resolveOptions = {
    extensions: ['.js', '.jsx', '.ts', '.tsx', '.json']
  };

  const resolveSync = _enhancedResolve.default.create.sync( // this._compilation is a deprecated API
  // However there seems to be no other way to access webpack's resolver
  // There is this.resolve, but it's asynchronous
  // Another option is to read the webpack.config.js, but it won't work for programmatic usage
  // This API is used by many loaders/plugins, so hope we're safe for a while
  ((_this$_compilation = this._compilation) === null || _this$_compilation === void 0 ? void 0 : _this$_compilation.options.resolve) ? { ...resolveOptions,
    alias: this._compilation.options.resolve.alias,
    modules: this._compilation.options.resolve.modules
  } : resolveOptions);

  let result;
  const originalResolveFilename = _babel.Module._resolveFilename;

  try {
    // Use webpack's resolution when evaluating modules
    _babel.Module._resolveFilename = (id, {
      filename
    }) => resolveSync(_path.default.dirname(filename), id);

    result = (0, _babel.transform)(content, {
      filename: _path.default.relative(process.cwd(), this.resourcePath),
      inputSourceMap: inputSourceMap !== null && inputSourceMap !== void 0 ? inputSourceMap : undefined,
      outputFilename,
      pluginOptions: rest,
      preprocessor
    });
  } finally {
    // Restore original behaviour
    _babel.Module._resolveFilename = originalResolveFilename;
  }

  if (result.cssText) {
    var _result$dependencies, _result$sourceMap;

    let {
      cssText
    } = result;

    if (sourceMap) {
      cssText += `/*# sourceMappingURL=data:application/json;base64,${Buffer.from(result.cssSourceMapText || '').toString('base64')}*/`;
    }

    if ((_result$dependencies = result.dependencies) === null || _result$dependencies === void 0 ? void 0 : _result$dependencies.length) {
      result.dependencies.forEach(dep => {
        try {
          const f = resolveSync(_path.default.dirname(this.resourcePath), dep);
          this.addDependency(f);
        } catch (e) {
          // eslint-disable-next-line no-console
          console.warn(`[linaria] failed to add dependency for: ${dep}`, e);
        }
      });
    } // Read the file first to compare the content
    // Write the new content only if it's changed
    // This will prevent unnecessary WDS reloads


    let currentCssText;

    try {
      currentCssText = _fs.default.readFileSync(outputFilename, 'utf-8');
    } catch (e) {// Ignore error
    }

    if (currentCssText !== cssText) {
      _mkdirp.default.sync(_path.default.dirname(outputFilename));

      _fs.default.writeFileSync(outputFilename, cssText);
    }

    this.callback(null, `${result.code}\n\nrequire(${_loaderUtils.default.stringifyRequest(this, outputFilename)});`, (_result$sourceMap = result.sourceMap) !== null && _result$sourceMap !== void 0 ? _result$sourceMap : undefined);
    return;
  }

  this.callback(null, result.code, (_result$sourceMap2 = result.sourceMap) !== null && _result$sourceMap2 !== void 0 ? _result$sourceMap2 : undefined);
}
//# sourceMappingURL=index.js.map